@@ -28,6 +28,7 @@
 #include "canvas.h"
 #include "edit.h"
 #include "accessibility.h"
+#include "fileops.h"
 
 #ifdef WITH_GPE
 #include "gpesettings_store.h"
@@ -57,6 +58,11 @@ GtkWidget *preferences_window;
 GtkListStore *alph_list_store;
 GtkListStore *colour_list_store;
 GladeXML *widgets;
+GtkWidget *open_filesel;
+GtkWidget *save_filesel;
+GtkWidget *save_and_quit_filesel;
+GtkWidget *import_filesel;
+GtkWidget *append_filesel;
 std::string alphabet;
 std::string colourscheme;
 char *system_data_dir;
@@ -64,6 +70,7 @@ char *user_data_dir;
 GtkWidget *train_dialog;
 GThread *trainthread;
 GAsyncQueue* trainqueue;
+ControlTree *controltree;
 
 bool controlmodeon=false;
 bool keyboardmodeon=false;
@@ -73,7 +80,9 @@ bool cyclickeyboardmodeon=false;
 
 button buttons[10];
 
+#ifndef GNOME_SPEECH
 #define _(_x) gettext(_x)
+#endif
 
 #define NO_PREV_POS -1
 
@@ -98,6 +107,7 @@ gboolean secondbox=FALSE;
 gboolean speakonstop=FALSE;
 gboolean training=FALSE;
 gboolean exiting=FALSE;
+gboolean direction=TRUE;
 
 gint dasherwidth, dasherheight;
 long yscale, mouseposstartdist=0;
@@ -117,6 +127,7 @@ gint fileencoding;
 
 gint outputcharacters;
 
+time_t lastdirection=0;
 time_t dasherstarttime;
 time_t starttime=0;
 time_t starttime2=0;
@@ -148,6 +159,7 @@ change_alphabet(gpointer alph)
   //  g_free(alph);
   g_async_queue_push(trainqueue,(void *)1);
   g_thread_exit(NULL);
+  return NULL;
 }
 
 
@@ -156,8 +168,6 @@ extern "C" void alphabet_select(GtkTreeSelection *selection, gpointer data)
   GtkTreeIter iter;
   GtkTreeModel *model;
   gchar *alph;
-  GdkCursor *waitcursor, *arrowcursor;
-  GtkWidget *preferences_window = GTK_WIDGET(data);
 
   if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
     gtk_tree_model_get(model, &iter, 0, &alph, -1);    
@@ -218,8 +228,6 @@ extern "C" void colour_select(GtkTreeSelection *selection, gpointer data)
   GtkTreeIter iter;
   GtkTreeModel *model;
   gchar *colour;
-  GdkCursor *waitcursor, *arrowcursor;
-  GtkWidget *preferences_window = GTK_WIDGET(data);
 
   if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
     gtk_tree_model_get(model, &iter, 0, &colour, -1);
@@ -278,7 +286,7 @@ generate_preferences(GtkWidget *widget, gpointer user_data) {
   }
   
   // Do the same for colours
-  colourtreeview = glade_xml_get_widget(widgets,"ColourTree");  
+  colourtreeview = glade_xml_get_widget(widgets,"ColorTree");  
 
   // Make sure that the colour tree is realized now as we'll need to do
   // stuff with it before it's actually displayed
@@ -359,24 +367,28 @@ button_preferences_show(GtkWidget *widget, gpointer user_data)
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(widgets,"spinbutton18")),buttons[9].y);
   gtk_window_set_transient_for(GTK_WINDOW(glade_xml_get_widget(widgets,"buttonprefs")),GTK_WINDOW(preferences_window));
   gtk_window_present(GTK_WINDOW(glade_xml_get_widget(widgets,"buttonprefs")));
+  return FALSE;
 }
 
 extern "C" gboolean
 button_preferences_hide(GtkWidget *widget, gpointer user_data)
 {
   gtk_widget_hide(glade_xml_get_widget(widgets,"buttonprefs"));
+  return FALSE;
 }
 
 extern "C" gboolean
 fontsel_hide(GtkWidget *widget, gpointer user_data)
 {
   gtk_widget_hide(GTK_WIDGET(dasher_fontselector));
+  return FALSE;
 }
 
 extern "C" gboolean
 edit_fontsel_hide(GtkWidget *widget, gpointer user_data)
 {
   gtk_widget_hide(GTK_WIDGET(edit_fontselector));
+  return FALSE;
 }
 
 extern "C" gboolean
@@ -457,58 +469,17 @@ button_coordinates_changed(GtkWidget *widget, gpointer user_data)
     buttons[9].y=value;
     set_long_option_callback("Button9Y",value);
   }
+  return FALSE;
 }
 
-extern "C" void 
-open_file (const char *myfilename)
+#if GTK_CHECK_VERSION(2,3,0)
+
+/* Fudge to avoid Glade complaining about being unable to find this signal 
+   handler */
+extern "C" void
+filesel_hide(GtkWidget *widget, gpointer user_data)
 {
-  int size;
-  gchar *buffer;
-  GtkWidget *error_dialog;
-
-  struct stat file_stat;
-  FILE *fp;
-  int pos = 0;
-
-  stat (myfilename, &file_stat);
-  fp = fopen (myfilename, "r");
-
-  if (fp==NULL || S_ISDIR(file_stat.st_mode)) {
-    error_dialog = gtk_message_dialog_new(GTK_WINDOW(window),GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK, "Could not open the file \"%s\".\n", myfilename);
-    gtk_dialog_set_default_response(GTK_DIALOG (error_dialog), GTK_RESPONSE_OK);
-    gtk_window_set_resizable(GTK_WINDOW(error_dialog), FALSE);
-    gtk_dialog_run(GTK_DIALOG(error_dialog));
-    gtk_widget_destroy(error_dialog);
-    return;
-  }
-
-  size=file_stat.st_size;
-  buffer = (gchar *) g_malloc (size);
-  fread (buffer, size, 1, fp);
-  fclose (fp);
-
-  dasher_clear();
-  
-  file_modified = TRUE;
-
-  if (size!=0) {
-    // Don't attempt to insert new text if the file is empty as it makes
-    // GTK cry
-    if (g_utf8_validate(buffer,size,NULL)==FALSE) {
-      // It's not UTF8, so we do the best we can...
-      gchar* buffer2=g_locale_to_utf8(buffer,size,NULL,NULL,NULL);
-      g_free(buffer);
-      buffer=buffer2;
-    }
-    gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER (the_text_buffer), buffer, size);
-    gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW (the_text_view),gtk_text_buffer_get_insert(GTK_TEXT_BUFFER(the_text_buffer)));
-  }
-
-  gtk_window_set_title(GTK_WINDOW(window), myfilename);
-  filename=myfilename;
-
-  dasher_start();
-  dasher_redraw();
+  return;
 }
 
 extern "C" void
@@ -524,109 +495,21 @@ select_new_file(GtkWidget *widget, gpointer user_data)
   dasher_pause(0,0);
 }
 
-extern "C" bool
-save_file_as (const char *filename, bool append)
-{
-  int opened=1;
-  gint length;
-  gchar *inbuffer,*outbuffer = NULL;
-  gsize bytes_read, bytes_written;
-  GError *error = NULL;
-  GIConv cd;
-  GtkWidget *error_dialog;
-  GtkTextIter *start, *end;
-
-  start = new GtkTextIter;
-  end = new GtkTextIter;
-
-  FILE *fp;
-
-  if (append == true) {
-    fp = fopen (filename, "a");
-
-    if (fp == NULL) {
-      opened = 0;
-    }
-  } else {
-    fp = fopen (filename, "w");
-    if (fp == NULL) {
-      opened = 0;
-    }
-  }
-
-  if (!opened) {
-    error_dialog = gtk_message_dialog_new(GTK_WINDOW(window),GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK, "Could not save the file \"%s\".\n", filename);
-    gtk_dialog_set_default_response(GTK_DIALOG (error_dialog), GTK_RESPONSE_OK);
-    gtk_window_set_resizable(GTK_WINDOW(error_dialog), FALSE);
-    gtk_dialog_run(GTK_DIALOG(error_dialog));
-    gtk_widget_destroy(error_dialog);
-    return false;
-  }
-
-  gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(the_text_buffer),start,0);
-  gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(the_text_buffer),end,-1);
-
-  inbuffer = gtk_text_iter_get_slice (start,end);
-
-  length = gtk_text_iter_get_offset(end)-gtk_text_iter_get_offset(start);
-  (void*)outbuffer = malloc((length+1)*sizeof(gchar));
-  memcpy((void*)outbuffer,(void*)inbuffer,length*sizeof(gchar));
-  outbuffer[length]=0;
-  g_free(inbuffer);
-  inbuffer=outbuffer;
-  outbuffer=NULL;
-
-  switch (fileencoding) {
-  case Opts::UserDefault:
-  case Opts::AlphabetDefault:
-    //FIXME - need to call GetAlphabetType and do appropriate stuff regarding the
-    // character set. Arguably we should always be saving in either UTF8 or the user's
-    // locale (which may, of course, be UTF8) because otherwise we're going to read
-    // in rubbish, and we shouldn't be encouraging weird codepage madness any further
-    //FIXME - error handling
-    outbuffer=g_locale_from_utf8(inbuffer,-1,&bytes_read,&bytes_written,&error);
-    if (outbuffer==NULL) {
-      // We can't represent the text in the current locale, so fall back to
-      // UTF-8
-      outbuffer=inbuffer;
-      bytes_written=length;
-    }
-  case Opts::UTF8:
-    outbuffer=inbuffer;
-    bytes_written=length;
-    break;
-  // Does /anyone/ want to save text files in UTF16?
-  // (in any case, my opinions regarding encouragement of data formats with
-  // endianness damage are almost certainly unprintable)
-
-  case Opts::UTF16LE:
-    cd=g_iconv_open("UTF16LE","UTF8");
-    outbuffer=g_convert_with_iconv(inbuffer,-1,cd,&bytes_read,&bytes_written,&error);
-    break;
-  case Opts::UTF16BE:
-    cd=g_iconv_open("UTF16BE","UTF8");
-    outbuffer=g_convert_with_iconv(inbuffer,-1,cd,&bytes_read,&bytes_written,&error);
-    break;
-  default:
-    outbuffer=inbuffer;
-    bytes_written=length;
-  }
-
-  fwrite(outbuffer,1,bytes_written,fp);
-  fclose (fp);
-
-  file_modified = FALSE;
-  gtk_window_set_title(GTK_WINDOW(window), filename);
-}
-
- 
 extern "C" void
 select_open_file(GtkWidget *widget, gpointer user_data)
 {
   GtkWidget* filesel = gtk_file_chooser_dialog_new (_("Select File"),GTK_WINDOW(window),GTK_FILE_CHOOSER_ACTION_OPEN,GTK_STOCK_OPEN,GTK_RESPONSE_ACCEPT,GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
 
+#ifdef GNOME_LIBS
+  gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(filesel),FALSE);
+#endif
+
   if (gtk_dialog_run (GTK_DIALOG(filesel)) == GTK_RESPONSE_ACCEPT) {
+#ifdef GNOME_LIBS
+    char *filename = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(filesel));
+#else
     char *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filesel));
+#endif
     open_file(filename);
     g_free(filename);
   }
@@ -637,9 +520,17 @@ extern "C" void
 select_save_file_as(GtkWidget *widget, gpointer user_data)
 {
   GtkWidget* filesel = gtk_file_chooser_dialog_new (_("Select File"),GTK_WINDOW(window),GTK_FILE_CHOOSER_ACTION_SAVE,GTK_STOCK_SAVE,GTK_RESPONSE_ACCEPT,GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
+
+#ifdef GNOME_LIBS
+  gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(filesel),FALSE);
+#endif
   
   if (gtk_dialog_run (GTK_DIALOG(filesel)) == GTK_RESPONSE_ACCEPT) {
+#ifdef GNOME_LIBS
+    char *filename = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(filesel));
+#else
     char *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filesel));
+#endif
     save_file_as(filename,FALSE);
     g_free(filename);
   }
@@ -650,9 +541,17 @@ extern "C" void
 select_append_file(GtkWidget *widget, gpointer user_data)
 {
   GtkWidget* filesel = gtk_file_chooser_dialog_new (_("Select File"),GTK_WINDOW(window),GTK_FILE_CHOOSER_ACTION_SAVE,GTK_STOCK_SAVE,GTK_RESPONSE_ACCEPT,GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
-  
+
+#ifdef GNOME_LIBS
+  gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(filesel),FALSE);
+#endif
+
   if (gtk_dialog_run (GTK_DIALOG(filesel)) == GTK_RESPONSE_ACCEPT) {
+#ifdef GNOME_LIBS
+    char *filename = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(filesel)); 
+#else
     char *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filesel));
+#endif
     save_file_as(filename,TRUE);
     g_free(filename);
   }
@@ -663,14 +562,148 @@ extern "C" void
 select_import_file(GtkWidget *widget, gpointer user_data)
 {
   GtkWidget* filesel = gtk_file_chooser_dialog_new (_("Select File"),GTK_WINDOW(window),GTK_FILE_CHOOSER_ACTION_OPEN,GTK_STOCK_OPEN,GTK_RESPONSE_ACCEPT,GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
+
+#ifdef GNOME_LIBS
+  gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(filesel),FALSE);
+#endif
   
   if (gtk_dialog_run (GTK_DIALOG(filesel)) == GTK_RESPONSE_ACCEPT) {
+#ifdef GNOME_LIBS
+    char *filename = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(filesel));
+#else
     char *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filesel));
+#endif
     load_training_file(filename);
     g_free(filename);
   }
   gtk_widget_destroy(filesel);
 }
+#else
+
+extern "C" void
+import_file_from_filesel ( GtkWidget *selector2, GtkFileSelection *selector )
+{
+  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(selector));
+  load_training_file(filename);  
+  filesel_hide(GTK_WIDGET(selector->ok_button),NULL);
+}
+
+extern "C" void               
+select_import_file(GtkWidget *widget, gpointer user_data)
+{
+  if (import_filesel==NULL) {
+    import_filesel = glade_xml_get_widget(widgets, "import_fileselector");
+    
+    g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (import_filesel)->ok_button),
+                      "clicked", G_CALLBACK (import_file_from_filesel), (gpointer) import_filesel);
+  }
+  
+  gtk_window_set_transient_for(GTK_WINDOW(import_filesel),GTK_WINDOW(window));
+  gtk_window_present (GTK_WINDOW(import_filesel));
+}
+
+extern "C" void
+open_file_from_filesel ( GtkWidget *selector2, GtkFileSelection *selector )
+{
+  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(selector));
+  filesel_hide(GTK_WIDGET(selector->ok_button),NULL);
+  open_file (filename);
+}
+
+extern "C" void
+select_open_file(GtkWidget *widget, gpointer user_data)
+{
+  if (open_filesel==NULL) {
+    open_filesel = glade_xml_get_widget(widgets, "open_fileselector");
+    g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (open_filesel)->ok_button),
+                      "clicked", G_CALLBACK (open_file_from_filesel), (gpointer) open_filesel);
+  }
+  gtk_window_set_transient_for(GTK_WINDOW(open_filesel),GTK_WINDOW(window));
+  gtk_window_present (GTK_WINDOW(open_filesel));
+}
+
+extern "C" void
+save_file_from_filesel ( GtkWidget *selector2, GtkFileSelection *selector )
+{
+  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(selector));
+  filesel_hide(GTK_WIDGET(selector->ok_button),NULL);
+  save_file_as(filename,FALSE);
+}
+
+extern "C" void
+save_file_from_filesel_and_quit ( GtkWidget *selector2, GtkFileSelection *selector )
+{
+  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(selector));
+  if (save_file_as(filename,FALSE)==false) {
+    return;
+  } else {
+    exiting=TRUE;
+    gtk_main_quit();
+  }
+}
+
+extern "C" void
+append_file_from_filesel ( GtkWidget *selector2, GtkFileSelection *selector )
+{
+  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(selector));
+  
+  save_file_as(filename,TRUE);
+  
+  filesel_hide(GTK_WIDGET(selector->ok_button),NULL);
+}
+
+extern "C" void
+select_save_file_as(GtkWidget *widget, gpointer user_data)
+{
+  if (save_filesel==NULL) {
+    save_filesel = glade_xml_get_widget(widgets, "save_fileselector");
+    g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (save_filesel)->ok_button),
+                      "clicked", G_CALLBACK (save_file_from_filesel), (gpointer) save_filesel);
+  }
+  
+  if (filename!=NULL)
+    gtk_file_selection_set_filename (GTK_FILE_SELECTION(save_filesel), filename);
+  gtk_window_set_transient_for(GTK_WINDOW(save_filesel),GTK_WINDOW(window));
+  gtk_window_present (GTK_WINDOW(save_filesel));
+}
+
+extern "C" void
+select_save_file_as_and_quit(GtkWidget *widget, gpointer user_data)
+{
+  if (save_and_quit_filesel==NULL) {
+    save_and_quit_filesel = glade_xml_get_widget(widgets, "save_and_quit_fileselector");
+    g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (save_and_quit_filesel)->ok_button),
+                      "clicked", G_CALLBACK (save_file_from_filesel_and_quit), (gpointer) save_and_quit_filesel);
+}
+  
+  if (filename!=NULL)
+    gtk_file_selection_set_filename (GTK_FILE_SELECTION(save_and_quit_filesel), filename);
+  gtk_window_set_transient_for(GTK_WINDOW(save_and_quit_filesel),GTK_WINDOW(window));
+  gtk_window_present (GTK_WINDOW(save_and_quit_filesel));
+}
+
+extern "C" void
+select_append_file(GtkWidget *widget, gpointer user_data)
+{
+  if (append_filesel==NULL) {
+    append_filesel = glade_xml_get_widget(widgets, "append_fileselector");
+    g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (append_filesel)->ok_button),
+                      "clicked", G_CALLBACK (append_file_from_filesel), (gpointer) append_filesel);
+  }
+  
+  gtk_window_set_transient_for(GTK_WINDOW(append_filesel),GTK_WINDOW(window));
+  gtk_window_present (GTK_WINDOW(append_filesel));
+}
+
+extern "C" void
+filesel_hide(GtkWidget *widget, gpointer user_data)
+{
+  // FIXME - uh. Yes. This works, but is it in any way guaranteed to?
+  // Of course, if glade let us set user_data stuff properly, this would
+  // be a lot easier
+  gtk_widget_hide(gtk_widget_get_parent(gtk_widget_get_parent(gtk_widget_get_parent(widget))));
+}
+#endif
 
 extern "C" void
 save_file (GtkWidget *widget, gpointer user_data)
@@ -704,7 +737,7 @@ extern "C" bool
 ask_save_before_exit(GtkWidget *widget, gpointer data)
 {
   GtkWidget *dialog = NULL;
-  quitting==TRUE;
+  quitting=TRUE;
 
   if (file_modified != FALSE) {
     // Ask whether to save the modified file, insert filename if it exists.
@@ -723,7 +756,7 @@ ask_save_before_exit(GtkWidget *widget, gpointer data)
       gtk_main_quit();
       break;
     case GTK_RESPONSE_CANCEL:
-      quitting==FALSE;
+      quitting=FALSE;
       gtk_widget_destroy (GTK_WIDGET(dialog));
       return true;
       break;
@@ -740,6 +773,7 @@ ask_save_before_exit(GtkWidget *widget, gpointer data)
     exiting=TRUE;
     gtk_main_quit();
   }
+  return false;
 }
 
 long get_time() {
@@ -787,12 +821,22 @@ timer_callback(gpointer data)
 	//	deletemenutree();
 	// And making bonobo calls from another thread is likely to lead to
 	// pain as well. It'd be nice to do this while training, but.
-	//	add_control_tree(gettree());
+	add_control_tree(controltree);
 
 	dasher_redraw();
       }
     }
-  if (!paused) {
+  
+    if (!paused) {
+    
+     if (eyetrackermode) {
+        if (direction==TRUE) {
+            dasher_set_parameter_int(INT_ONEBUTTON, 150);
+        }
+        if (direction==FALSE) {
+            dasher_set_parameter_int(INT_ONEBUTTON, -150); 
+        }
+    }
     int x;
     int y;
     
@@ -970,12 +1014,13 @@ edit_button_release_event (GtkWidget *widget, GdkEventButton *event, gpointer da
     dasher_redraw();
     return FALSE;
   }
+  return FALSE;
 }
 
 extern "C" gint
 key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
-  int i,width,height;
+  int width,height;
   if (event->type != GDK_KEY_PRESS)
     return FALSE;
 
@@ -1159,11 +1204,17 @@ button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer data)
     return FALSE;
 #endif
 
+
+
   // CJB,  2003-08.  If we have a selection, replace it with the new input.
   // This code is duplicated in key_press_event.
   if (gtk_text_buffer_get_selection_bounds (the_text_buffer, NULL, NULL))
     gtk_text_buffer_cut_clipboard(the_text_buffer, the_text_clipboard, TRUE);
-  
+
+  // CJB.  2004-07.
+  // One-button mode; change direction on mouse click.
+  direction=!direction;
+
   if (startleft == TRUE) {
     stop();
   }
@@ -1196,7 +1247,7 @@ void interface_setup(GladeXML *xml) {
 
   // What's this doing here? I'm sure we ought to just be using whatever
   // the core provides us with
-  float initial_bitrate = 3.0;
+  // float initial_bitrate = 3.0;
 
   the_canvas=glade_xml_get_widget(xml, "the_canvas");
 
@@ -1207,15 +1258,6 @@ void interface_setup(GladeXML *xml) {
   the_text_view=glade_xml_get_widget(xml, "the_text_view");
   toolbar=glade_xml_get_widget(xml, "toolbar");
 
-  if (textentry==TRUE) {
-    gtk_paned_set_position(GTK_PANED(glade_xml_get_widget(widgets,"vpaned1")),0);
-    gtk_widget_hide(text_scrolled_window);
-    keyboardmodeon=true;
-    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(glade_xml_get_widget(widgets,"keyboardmode")), true);
-    gtk_widget_set_sensitive(glade_xml_get_widget(widgets,"keyboardmode"),false);
-    gtk_widget_hide(toolbar);
-  }
-
 #ifndef GNOME_SPEECH
   // This ought to be greyed out if not built with speech support
   gtk_widget_set_sensitive(glade_xml_get_widget(widgets,"speakbutton"),false);
@@ -1302,6 +1344,7 @@ void
 interface_late_setup() {
   // Stuff that needs to be done after the core has
   // set itself up
+  dasher_set_parameter_int(INT_ONEBUTTON, 0);
   alphabet=dasher_get_current_alphabet();
   colourscheme=dasher_get_current_colours();
   generate_preferences(NULL,NULL);
@@ -1359,6 +1402,7 @@ open_window(GladeXML *xml) {
   // We need to monitor the text buffer for mark_set in order to get
   // signals when the cursor is moved
   g_signal_connect(G_OBJECT(the_text_buffer), "mark_set", G_CALLBACK(edit_button_release_event), NULL);
+
 }
 
 extern "C" void choose_filename() {
@@ -1366,19 +1410,30 @@ extern "C" void choose_filename() {
     // Build a filename based on the current time and date
     tm *t_struct;
     time_t ctime;
+    char *cwd;
+    char *tbuffer;
+
+    cwd=(char *)malloc(1024*sizeof(char));
+    tbuffer=(char *)malloc(1024*sizeof(char));
     
     ctime = time( NULL );
     
     t_struct= localtime( &ctime );
     
-    char tbuffer[256];
     
     snprintf( tbuffer, 256, "dasher-%d%d%d-%d%d.txt", (t_struct->tm_year+1900), (t_struct->tm_mon+1), t_struct->tm_mday, t_struct->tm_hour, t_struct->tm_min);
     
-    filename = g_strdup (tbuffer);
+    if (filename) {
+      g_free((void *)filename);
+    }
+    getcwd(cwd,1024);
+    filename = g_build_path("/",cwd,tbuffer,NULL);
     gtk_window_set_title(GTK_WINDOW(window), filename);
   } else {
     gtk_window_set_title(GTK_WINDOW(window), "Dasher");
+    if (filename) {
+      g_free((void *)filename);
+    }
     filename = NULL;
   }
 }
@@ -1558,11 +1613,18 @@ extern "C" void button_cyclical_mode(GtkWidget *widget, gpointer user_data)
   set_bool_option_callback("Cyclicalbuttons",cyclickeyboardmodeon);
 }
 
+extern "C" void show_help(GtkWidget *widget, gpointer user_data)
+{
+#ifdef GNOME_LIBS
+  gnome_help_display_desktop(NULL, "dasher", "dasher", NULL, NULL);
+#endif
+}
+
 extern "C" void about_dasher(GtkWidget *widget, gpointer user_data)
 {
 #ifdef GNOME_LIBS
   // Give them a lovely Gnome-style about box
-  GdkPixbuf* pixbuf = NULL;
+  //GdkPixbuf* pixbuf = NULL;
 
   // In alphabetical order
   gchar *authors[] = {
@@ -1736,6 +1798,8 @@ void parameter_string_callback( string_param p, const char *value )
 	editfont="Sans 10";
       }
       break;
+    default:
+      break;
     }
 }
 
@@ -1852,6 +1916,8 @@ void parameter_int_callback( int_param p, long int value )
 #endif
       dasher_redraw();
       break;
+    default:
+      break;
     }
 }
 
@@ -1930,7 +1996,7 @@ void parameter_bool_callback( bool_param p, bool value )
       break;
     case BOOL_KEYBOARDCONTROL:
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(glade_xml_get_widget(widgets,"keyboardbutton")), value);
-      keyboardcontrol=value;
+      keyboardcontrol=value;      
       break;
     case BOOL_WINDOWPAUSE:
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(glade_xml_get_widget(widgets,"winpausebutton")), value);
@@ -1959,6 +2025,8 @@ void parameter_bool_callback( bool_param p, bool value )
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(glade_xml_get_widget(widgets,"speakbutton")), value);
       speakonstop=value;
       break;
+    default:
+      break;
     }
 }
 
@@ -1994,7 +2062,7 @@ void stop() {
     }
     if (timedata==TRUE) {
       // Just a debugging thing, output to the console
-      printf(_("%d characters output in %d seconds\n"),outputcharacters,
+      printf(_("%d characters output in %ld seconds\n"),outputcharacters,
 	     time(NULL)-dasherstarttime);
       outputcharacters=0;
     }
@@ -2014,7 +2082,7 @@ void scan_alphabet_files()
   alphabetglob=g_pattern_spec_new("alphabet*xml");
   directory = g_dir_open(system_data_dir,0,NULL);
 
-  while(filename=g_dir_read_name(directory)) {
+  while((filename=g_dir_read_name(directory))) {
     if (alphabet_filter(filename)) {
       add_alphabet_filename(filename);
     }
@@ -2022,7 +2090,7 @@ void scan_alphabet_files()
 
   directory = g_dir_open(user_data_dir,0,NULL);
 
-  while(filename=g_dir_read_name(directory)) {
+  while((filename=g_dir_read_name(directory))) {
     if (alphabet_filter(filename)) {
       add_alphabet_filename(filename);
     }
@@ -2036,7 +2104,7 @@ void scan_colour_files()
   colourglob=g_pattern_spec_new("colour*xml");
   directory = g_dir_open(system_data_dir,0,NULL);
 
-  while(filename=g_dir_read_name(directory)) {
+  while((filename=g_dir_read_name(directory))) {
     if (colour_filter(filename)) {
       add_colour_filename(filename);
     }
@@ -2044,7 +2112,7 @@ void scan_colour_files()
 
   directory = g_dir_open(user_data_dir,0,NULL);
 
-  while(filename=g_dir_read_name(directory)) {
+  while((filename=g_dir_read_name(directory))) {
     if (colour_filter(filename)) {
       add_colour_filename(filename);
     }
