@@ -15,13 +15,18 @@ void setup_speech() {
 
   Bonobo_ServerInfoList *servers;
   Bonobo_ServerInfo *info;
-  CORBA_Object rv;
+  CORBA_Object rv=NULL;
 
-  servers = bonobo_activation_query (
-	     "repo_ids.has ('IDL:GNOME/Speech/SynthesisDriver:0.2')",
-	     NULL, &ev);
+  servers = bonobo_activation_query ("repo_ids.has_one(['IDL:GNOME/Speech/SynthesisDriver:0.2','IDL:GNOME/Speech/SynthesisDriver:0.3'])", NULL, &ev);
 
-  for (int i=0; i<servers->_length; i++) 
+  if (servers->_length==0) {
+    speaker=NULL;
+    printf(_("Unable to initialize speech support\n"));
+    fflush(stdout);
+    return;
+  }
+
+  for (unsigned int i=0; i<servers->_length; i++) 
     {
       CORBA_exception_init (&ev);
 
@@ -41,12 +46,17 @@ void setup_speech() {
 
   voices = GNOME_Speech_SynthesisDriver_getAllVoices (rv, &ev);
 
+  if (voices==NULL || BONOBO_EX (&ev)) {
+    printf(_("Unable to initialize voices"));
+    speaker=NULL;
+    return;
+  }
+
   speaker = GNOME_Speech_SynthesisDriver_createSpeaker (rv,
                                                         &voices->_buffer[0],
                                                         &ev);
 
   GNOME_Speech_ParameterList *list;
-  int i;
   list = GNOME_Speech_Speaker_getSupportedParameters (speaker, &ev);
   if (BONOBO_EX (&ev) || list->_length == 0)
     {
@@ -55,7 +65,7 @@ void setup_speech() {
       printf(_("Warning: unable to set speech parameters\n"));
       return;
     }
-  for (i = 0; i < list->_length; i++)
+  for (unsigned i = 0; i < list->_length; i++)
     {
       GNOME_Speech_Parameter *p = &(list->_buffer[i]);
       if (!strcmp (p->name, "rate")) {
@@ -74,7 +84,9 @@ void teardown_speech() {
 }
 
 void SPEAK_DAMN_YOU(std::string *speech) {
-  GNOME_Speech_Speaker_say (speaker, speech->c_str(), &ev);
+  if (speaker!=NULL) {
+    GNOME_Speech_Speaker_say (speaker, speech->c_str(), &ev);
+  }
 }
 
 #else
